﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Problem
{
    public class Order
    {
        public int Id { get; private set; }
        public OrderStatus OrderStatus { get; private set; }

        private readonly List<OrderLine> _orderlines = new List<OrderLine>();
        private readonly ShippingDetails _shippingDetails;
        private readonly InvoiceDetails _invoiceDetails;

        private readonly ShippingGateway _shippingGateway;

        public Order(IEnumerable<OrderLine> orderLines, ShippingDetails shippingDetails, InvoiceDetails invoiceDetails)
        {
            Id = 0;
            OrderStatus = OrderStatus.Concept;

            _orderlines.AddRange(orderLines);
            _shippingDetails = shippingDetails;
            _invoiceDetails = invoiceDetails;

            _shippingGateway = new ShippingGateway();
        }

        public void AddOrderLine(string description, decimal price)
        {
            if (OrderStatus != OrderStatus.Concept)
            {
                throw new InvalidOperationException("An order can only be altered when it is still in concept.");
            }

            _orderlines.Add(new OrderLine(description, price));
        }

        public void RemoveOrderLine(Guid orderlineIdentifier)
        {
            if (OrderStatus != OrderStatus.Concept)
            {
                throw new InvalidOperationException("An order can only be altered when it is still in concept.");
            }

            _orderlines.RemoveAll(line => line.Identifier == orderlineIdentifier);
        }

        public void Cancel()
        {
            if (OrderStatus != OrderStatus.Concept)
            {
                throw new InvalidOperationException("An order can only be canceled when it is still in concept.");
            }

            OrderStatus = OrderStatus.Canceled;
        }

        public void FinalizeOrder()
        {
            if (OrderStatus != OrderStatus.Concept)
            {
                throw new InvalidOperationException("An order can only be finalized when it is still in concept.");
            }

            CalculateDeliveryCosts();
            CalculatePaymentcosts();
            OrderStatus = OrderStatus.Ordered;
        }

        public void Pay()
        {
            if (OrderStatus != OrderStatus.Ordered)
            {
                throw new InvalidOperationException("An order can only be payed when it is final.");
            }
        }

        public void ShipOrder()
        {
            if (OrderStatus != OrderStatus.Paid)
            {
                throw new InvalidOperationException("An order can only be shipped when it is payed.");
            }

            VerifyShippingDetails();

            var shippingDate = _shippingGateway.DetermineShippingDate(this);
            _shippingDetails.SetShippingDate(shippingDate);


            OrderStatus = OrderStatus.Shipped;
        }

        // etc etc etc
        // Irrelevant private's below..

        private void CalculatePaymentcosts()
        {
            _orderlines.Add(new OrderLine("Betaalkosten", 2.95m));
        }

        private void CalculateDeliveryCosts()
        {
            _orderlines.Add(new OrderLine("Bezorgkosten", 5.95m));
        }

        private void VerifyShippingDetails()
        {
            ; // Let's pretend that everything is in order (pun intended)
        }

    }
}
