﻿using System;
using System.Collections.Generic;

namespace StepI
{
    public abstract class OrderState
    {
        public int Id { get; private set; }
        public OrderStatus OrderStatus { get; protected set; }

        protected readonly List<OrderLine> _orderlines = new List<OrderLine>();
        protected readonly ShippingDetails _shippingDetails;
        protected readonly InvoiceDetails _invoiceDetails;

        protected readonly ShippingGateway _shippingGateway;

        public OrderState(IEnumerable<OrderLine> orderLines, ShippingDetails shippingDetails, InvoiceDetails invoiceDetails)
        {
            Id = 0;
            OrderStatus = OrderStatus.Concept;

            _orderlines.AddRange(orderLines);
            _shippingDetails = shippingDetails;
            _invoiceDetails = invoiceDetails;

            _shippingGateway = new ShippingGateway();
        }

        public abstract OrderState Cancel();

        public abstract OrderState FinalizeOrder();

        public abstract OrderState Pay();

        public abstract OrderState ShipOrder();

        public abstract OrderState RemoveOrderLine(Guid orderlineIdentifier);

        public abstract OrderState AddOrderLine(string description, decimal price);
    }
}
