﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace StepI
{
    public class ToRemove : OrderState
    {
        public ToRemove(IEnumerable<OrderLine> orderLines, ShippingDetails shippingDetails, InvoiceDetails invoiceDetails)
            : base(orderLines, shippingDetails, invoiceDetails)
        {
        }

        public override OrderState AddOrderLine(string description, decimal price)
        {
            if (OrderStatus != OrderStatus.Concept)
            {
                throw new InvalidOperationException("An order can only be altered when it is still in concept.");
            }

            _orderlines.Add(new OrderLine(description, price));

            return this;
        }

        public override OrderState RemoveOrderLine(Guid orderlineIdentifier)
        {
            if (OrderStatus != OrderStatus.Concept)
            {
                throw new InvalidOperationException("An order can only be altered when it is still in concept.");
            }

            _orderlines.RemoveAll(line => line.Identifier == orderlineIdentifier);

            return this;
        }

        public override OrderState Cancel()
        {
            if (OrderStatus != OrderStatus.Concept)
            {
                throw new InvalidOperationException("An order can only be canceled when it is still in concept.");
            }

            OrderStatus = OrderStatus.Canceled;

            return this;
        }

        public override OrderState FinalizeOrder()
        {
            if (OrderStatus != OrderStatus.Concept)
            {
                throw new InvalidOperationException("An order can only be finalized when it is still in concept.");
            }

            CalculateDeliveryCosts();
            CalculatePaymentcosts();
            OrderStatus = OrderStatus.Ordered;

            return this;
        }

        public override OrderState Pay()
        {
            if (OrderStatus != OrderStatus.Ordered)
            {
                throw new InvalidOperationException("An order can only be payed when it is final.");
            }

            return this;
        }

        public override OrderState ShipOrder()
        {
            if (OrderStatus != OrderStatus.Paid)
            {
                throw new InvalidOperationException("An order can only be shipped when it is payed.");
            }

            VerifyShippingDetails();

            var shippingDate = _shippingGateway.DetermineShippingDate(this);
            _shippingDetails.SetShippingDate(shippingDate);


            OrderStatus = OrderStatus.Shipped;
            return this;
        }

        // etc etc etc
        // Irrelevant private's below..

        private void CalculatePaymentcosts()
        {
            _orderlines.Add(new OrderLine("Betaalkosten", 2.95m));
        }

        private void CalculateDeliveryCosts()
        {
            _orderlines.Add(new OrderLine("Bezorgkosten", 5.95m));
        }

        private void VerifyShippingDetails()
        {
            ; // Let's pretend that everything is in order (pun intended)
        }

    }
}
