﻿using System;
using System.Collections.Generic;

namespace StepIII.OrderStates
{
    internal class Concept : OrderState
    {
        public Concept(IEnumerable<OrderLine> orderLines, ShippingDetails shippingDetails, InvoiceDetails invoiceDetails)
            : base(orderLines, shippingDetails, invoiceDetails)
        {
        }

        public override OrderState AddOrderLine(string description, decimal price)
        {
            _orderlines.Add(new OrderLine(description, price));

            return this;
        }

        public override OrderState Cancel()
        {
            return new Canceled(_orderlines, _shippingDetails, _invoiceDetails);
        }

        public override OrderState FinalizeOrder()
        {
            CalculateDeliveryCosts();
            CalculatePaymentcosts();

            return new Ordered(_orderlines, _shippingDetails, _invoiceDetails);
        }

        private void CalculatePaymentcosts()
        {
            _orderlines.Add(new OrderLine("Betaalkosten", 2.95m));
        }

        private void CalculateDeliveryCosts()
        {
            _orderlines.Add(new OrderLine("Bezorgkosten", 5.95m));
        }

        public override OrderState Pay()
        {
            throw new InvalidOperationException("An order can only be payed when it is final.");
        }

        public override OrderState RemoveOrderLine(Guid orderlineIdentifier)
        {
            _orderlines.RemoveAll(line => line.Identifier == orderlineIdentifier);

            return this;
        }

        public override OrderState ShipOrder()
        {
            throw new InvalidOperationException("An order can only be shipped when it is payed.");
        }

    }
}
